<?php
/**
 * Plugin Name: Honeypot Anti-Spam
 * Description: Blockiert Spam-Bots durch ein verstecktes Honeypot-Feld.
 * Version: 1.0
 * Author: Manuel Hanetzok
 * Author URL: https://hanetzok-medien.de/
 * License: GPL2
 */

if (!defined('ABSPATH')) {
    exit;
}

// -------------------------
// 1) Plugin-Konstanten
// -------------------------
define('HAS_PLUGIN_BASENAME', plugin_basename(__FILE__));
define('HAS_PLUGIN_DIR', plugin_dir_path(__FILE__));
define('HAS_PLUGIN_URL', plugin_dir_url(__FILE__));

// -------------------------
// 2) Standardwerte
// -------------------------
register_activation_hook(__FILE__, 'has_set_default_options');
function has_set_default_options() {
    if (!get_option('has_honeypot_field_name')) {
        update_option('has_honeypot_field_name', 'hp_field');
    }
    if (!get_option('has_honeypot_error_message')) {
        update_option('has_honeypot_error_message', 'Spam erkannt. Formular konnte nicht gesendet werden.');
    }
    if (!get_option('has_spam_block_count')) {
        update_option('has_spam_block_count', 0);
    }
}

// -------------------------
// 3) Filter/Hooks für Honeypot
// -------------------------
// a) WP-Kommentare
add_filter('preprocess_comment', 'has_check_comment_honeypot');
function has_check_comment_honeypot($commentdata) {
    $field_name = get_option('has_honeypot_field_name');
    if (!empty($_POST[$field_name])) {
        has_increment_spam_count();
        wp_die(get_option('has_honeypot_error_message'));
    }
    return $commentdata;
}

// b) WP-Registrierung
add_filter('registration_errors', 'has_check_registration_honeypot', 10, 3);
function has_check_registration_honeypot($errors, $sanitized_user_login, $user_email) {
    $field_name = get_option('has_honeypot_field_name');
    if (!empty($_POST[$field_name])) {
        has_increment_spam_count();
        $errors->add('honeypot_error', get_option('has_honeypot_error_message'));
    }
    return $errors;
}

// c) Contact Form 7
// Verstecktes Feld einfügen
add_filter('wpcf7_form_hidden_fields', 'has_wpcf7_add_honeypot_field');
function has_wpcf7_add_honeypot_field($hidden_fields) {
    $field_name = get_option('has_honeypot_field_name');
    $hidden_fields[$field_name] = '';
    return $hidden_fields;
}

// Eingaben prüfen
add_action('wpcf7_before_send_mail', 'has_wpcf7_check_honeypot_field');
function has_wpcf7_check_honeypot_field($contact_form) {
    $submission = WPCF7_Submission::get_instance();
    if (!$submission) {
        return;
    }

    $data = $submission->get_posted_data();
    $field_name = get_option('has_honeypot_field_name');
    if (!empty($data[$field_name])) {
        has_increment_spam_count();
        // Stoppe Mailversand
        $mail = $contact_form->prop('mail');
        $mail['subject'] = 'SPAM: ' . $mail['subject'];
        $contact_form->set_properties(['mail' => $mail]);
        // Beispielhaft E-Mail anpassen oder Abbruch
        add_filter('wpcf7_skip_mail', '__return_true');
        $submission->set_status('aborted');
        $submission->set_response(get_option('has_honeypot_error_message'));
    }
}

// d) WPForms
// Verstecktes Feld in Formulare integrieren
add_action('wpforms_frontend_output', 'has_wpforms_add_honeypot_field', 10, 5);
function has_wpforms_add_honeypot_field($form_data, $form, $container, $css, $head) {
    $field_name = get_option('has_honeypot_field_name');
    echo '<input type="hidden" name="' . esc_attr($field_name) . '" value="" />';
}

// Eingaben prüfen
add_action('wpforms_process', 'has_wpforms_check_honeypot_field', 10, 3);
function has_wpforms_check_honeypot_field($fields, $entry, $form_data) {
    $field_name = get_option('has_honeypot_field_name');
    if (!empty($_POST[$field_name])) {
        has_increment_spam_count();
        wp_die(get_option('has_honeypot_error_message'));
    }
}

// -------------------------
// 4) Spam-Counter-Funktion
// -------------------------
function has_increment_spam_count() {
    $count = (int) get_option('has_spam_block_count');
    $count++;
    update_option('has_spam_block_count', $count);
}

// -------------------------
// 5) Backend-Menü
// -------------------------
add_action('admin_menu', 'has_add_admin_menu');
function has_add_admin_menu() {
    add_menu_page(
        'Honeypot Anti-Spam',
        'Honeypot Anti-Spam',
        'manage_options',
        'honeypot-anti-spam-settings',
        'has_render_admin_settings',
        'dashicons-shield-alt'
    );
}

// -------------------------
// 6) Einstellungen & Statistik
// -------------------------
function has_render_admin_settings() {
    if (!current_user_can('manage_options')) {
        return;
    }

    // Einstellungen speichern
    if (isset($_POST['has_submit_settings'])) {
        check_admin_referer('has_save_settings');
        update_option('has_honeypot_field_name', sanitize_text_field($_POST['has_honeypot_field_name']));
        update_option('has_honeypot_error_message', sanitize_text_field($_POST['has_honeypot_error_message']));
        echo '<div class="updated"><p>Einstellungen gespeichert.</p></div>';
    }

    $field_name = get_option('has_honeypot_field_name');
    $error_msg = get_option('has_honeypot_error_message');
    $spam_count = (int) get_option('has_spam_block_count');
    ?>
    <div class="wrap">
        <h1>Honeypot Anti-Spam Einstellungen</h1>
        <form method="post" action="">
            <?php wp_nonce_field('has_save_settings'); ?>
            <table class="form-table">
                <tr>
                    <th><label for="has_honeypot_field_name">Honey Pot Field Name</label></th>
                    <td>
                        <input type="text"
                               name="has_honeypot_field_name"
                               value="<?php echo esc_attr($field_name); ?>"
                               class="regular-text"/>
                    </td>
                </tr>
                <tr>
                    <th><label for="has_honeypot_error_message">Honey Pot Error Message</label></th>
                    <td>
                        <input type="text"
                               name="has_honeypot_error_message"
                               value="<?php echo esc_attr($error_msg); ?>"
                               class="regular-text"/>
                    </td>
                </tr>
                <tr>
                    <th>Statistik</th>
                    <td>
                        <p>Insgesamt geblockte Spam-Angriffe: <strong><?php echo $spam_count; ?></strong></p>
                    </td>
                </tr>
            </table>
            <?php submit_button('Einstellungen speichern', 'primary', 'has_submit_settings'); ?>
        </form>
    </div>
    <?php
}
